@@ -39,7 +39,7 @@
  * |[
  * gst-launch-1.0 -v videotestsrc pattern=ball name=left \
  *     videotestsrc name=right glstereomix name=mix \
- *     left. ! video/x-raw,width=640,height=480! glupload ! mix.  \
+ *     left. ! vid/x-raw,width=640,height=480! glupload ! mix.  \
  *     right. ! video/x-raw,width=640,height=480! glupload ! mix.  \
  *     mix. ! video/x-raw'(memory:GLMemory)',multiview-mode=side-by-side ! \
  *     queue ! glimagesink output-multiview-mode=side-by-side
@@ -61,9 +61,7 @@
 #include "config.h"
 #endif
 
-#include "gstglelements.h"
 #include "gstglstereomix.h"
-#include <gst/gl/gl.h>
 
 #define GST_CAT_DEFAULT gst_gl_stereo_mix_debug
 GST_DEBUG_CATEGORY (gst_gl_stereo_mix_debug);
@@ -87,8 +85,6 @@ static void gst_gl_stereo_mix_child_proxy_init (gpointer g_iface,
 G_DEFINE_TYPE_WITH_CODE (GstGLStereoMix, gst_gl_stereo_mix, GST_TYPE_GL_MIXER,
     G_IMPLEMENT_INTERFACE (GST_TYPE_CHILD_PROXY,
         gst_gl_stereo_mix_child_proxy_init));
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (glstereomix, "glstereomix",
-    GST_RANK_NONE, GST_TYPE_GL_STEREO_MIX, gl_element_init (plugin));
 
 static GstCaps *_update_caps (GstVideoAggregator * vagg, GstCaps * caps);
 static gboolean _negotiated_caps (GstAggregator * aggregator, GstCaps * caps);
@@ -182,9 +178,9 @@ gst_gl_stereo_mix_class_init (GstGLStereoMixClass * klass)
   gobject_class->get_property = gst_gl_stereo_mix_get_property;
   gobject_class->set_property = gst_gl_stereo_mix_set_property;
 
-  gst_element_class_set_static_metadata (element_class,
-      "OpenGL stereo video combiner", "Filter/Effect/Video",
-      "OpenGL stereo video combiner", "Jan Schmidt <jan@centricular.com>");
+  gst_element_class_set_metadata (element_class, "OpenGL stereo video combiner",
+      "Filter/Effect/Video", "OpenGL stereo video combiner",
+      "Jan Schmidt <jan@centricular.com>");
 
   g_object_class_install_property (gobject_class, PROP_DOWNMIX_MODE,
       g_param_spec_enum ("downmix-mode", "Mode for mono downmixed output",
@@ -214,9 +210,6 @@ gst_gl_stereo_mix_class_init (GstGLStereoMixClass * klass)
 
   base_mix_class->supported_gl_api =
       GST_GL_API_GLES2 | GST_GL_API_OPENGL | GST_GL_API_OPENGL3;
-
-  gst_type_mark_as_plugin_api (GST_TYPE_GL_STEREO_DOWNMIX, 0);
-  gst_type_mark_as_plugin_api (GST_TYPE_GL_STEREO_MIX_PAD, 0);
 }
 
 static void
@@ -370,7 +363,7 @@ gst_gl_stereo_mix_aggregate_frames (GstVideoAggregator * vagg,
   GstGLStereoMix *mix = GST_GL_STEREO_MIX (vagg);
   /* If we're operating in frame-by-frame mode, push
    * the primary view now, and let the parent class
-   * push the remaining auxiliary view */
+   * push the remaining auxilliary view */
   if (GST_VIDEO_INFO_MULTIVIEW_MODE (&vagg->info) ==
       GST_VIDEO_MULTIVIEW_MODE_FRAME_BY_FRAME) {
     /* Transfer the timestamps video-agg put on the aux buffer */
@@ -605,8 +598,7 @@ _update_caps (GstVideoAggregator * vagg, GstCaps * caps)
   blend_caps = gst_video_info_to_caps (mix_info);
 
   gst_caps_set_features (blend_caps, 0,
-      gst_caps_features_new_single_static_str
-      (GST_CAPS_FEATURE_MEMORY_GL_MEMORY));
+      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_GL_MEMORY));
 
   tmp_caps = get_converted_caps (GST_GL_STEREO_MIX (vagg), blend_caps);
   gst_caps_unref (blend_caps);
@@ -641,13 +633,11 @@ _negotiated_caps (GstAggregator * agg, GstCaps * caps)
 
   in_caps = gst_video_info_to_caps (&mix->mix_info);
   gst_caps_set_features (in_caps, 0,
-      gst_caps_features_new_single_static_str
-      (GST_CAPS_FEATURE_MEMORY_GL_MEMORY));
+      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_GL_MEMORY));
   gst_caps_set_simple (in_caps, "texture-target", G_TYPE_STRING,
       GST_GL_TEXTURE_TARGET_2D_STR, NULL);
 
   gst_gl_view_convert_set_caps (mix->viewconvert, in_caps, caps);
-  gst_caps_unref (in_caps);
 
   return TRUE;
 }
@@ -710,10 +700,6 @@ gst_gl_stereo_mix_process_frames (GstGLStereoMix * mixer)
   else
     views = 1;
 
-  /* We can configure the view_converter now */
-  gst_gl_view_convert_set_context (mixer->viewconvert,
-      GST_GL_BASE_MIXER (mixer)->context);
-
   if (gst_gl_view_convert_submit_input_buffer (mixer->viewconvert,
           FALSE, inbuf) != GST_FLOW_OK)
     return FALSE;
