@@ -36,62 +36,53 @@
 
 /**
  * SECTION:gstaudiocdsrc
+ * @title: GstAudioCdSrc
  * @short_description: Base class for Audio CD sources
  *
- * <para>
  * Provides a base class for CD digital audio (CDDA) sources, which handles
  * things like seeking, querying, discid calculation, tags, and buffer
  * timestamping.
- * </para>
- * <refsect2>
- * <title>Using GstAudioCdSrc-based elements in applications</title>
- * <para>
+ *
+ * ## Using GstAudioCdSrc-based elements in applications
+ *
  * GstAudioCdSrc registers two #GstFormat<!-- -->s of its own, namely
  * the "track" format and the "sector" format. Applications will usually
  * only find the "track" format interesting. You can retrieve that #GstFormat
  * for use in seek events or queries with gst_format_get_by_nick("track").
- * </para>
- * <para>
+ *
  * In order to query the number of tracks, for example, an application would
  * set the CDDA source element to READY or PAUSED state and then query the
  * the number of tracks via gst_element_query_duration() using the track
  * format acquired above. Applications can query the currently playing track
  * in the same way.
- * </para>
- * <para>
+ *
  * Alternatively, applications may retrieve the currently playing track and
  * the total number of tracks from the taglist that will posted on the bus
  * whenever the CD is opened or the currently playing track changes. The
  * taglist will contain GST_TAG_TRACK_NUMBER and GST_TAG_TRACK_COUNT tags.
- * </para>
- * <para>
+ *
  * Applications playing back CD audio using playbin and cdda://n URIs should
  * issue a seek command in track format to change between tracks, rather than
  * setting a new cdda://n+1 URI on playbin (as setting a new URI on playbin
  * involves closing and re-opening the CD device, which is much much slower).
- * </para>
- * <refsect2>
- * </refsect2>
- * <title>Tags and meta-information</title>
- * <para>
+ *
+ * ## Tags and meta-information
+ *
  * CDDA sources will automatically emit a number of tags, details about which
  * can be found in the libgsttag documentation. Those tags are:
  * #GST_TAG_CDDA_CDDB_DISCID, #GST_TAG_CDDA_CDDB_DISCID_FULL,
  * #GST_TAG_CDDA_MUSICBRAINZ_DISCID, #GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL,
  * among others.
- * </para>
- * </refsect2>
- * <refsect2>
- * <title>Tracks and Table of Contents (TOC)</title>
- * <para>
+ *
+ * ## Tracks and Table of Contents (TOC)
+ *
  * Applications will be informed of the available tracks via a TOC message
  * on the pipeline's #GstBus. The #GstToc will contain a #GstTocEntry for
  * each track, with information about each track. The duration for each
  * track can be retrieved via the #GST_TAG_DURATION tag from each entry's
  * tag list, or calculated via gst_toc_entry_get_start_stop_times().
  * The track entries in the TOC will be sorted by track number.
- * </para>
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -188,6 +179,7 @@ static GstIndex *gst_audio_cd_src_get_index (GstElement * src);
 
 #define gst_audio_cd_src_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstAudioCdSrc, gst_audio_cd_src, GST_TYPE_PUSH_SRC,
+    G_ADD_PRIVATE (GstAudioCdSrc)
     G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
         gst_audio_cd_src_uri_handler_init));
 
@@ -209,24 +201,6 @@ GST_STATIC_PAD_TEMPLATE ("src",
 static GstFormat track_format;
 static GstFormat sector_format;
 
-GType
-gst_audio_cd_src_mode_get_type (void)
-{
-  static GType mode_type;       /* 0 */
-  static const GEnumValue modes[] = {
-    {GST_AUDIO_CD_SRC_MODE_NORMAL, "Stream consists of a single track",
-        "normal"},
-    {GST_AUDIO_CD_SRC_MODE_CONTINUOUS, "Stream consists of the whole disc",
-        "continuous"},
-    {0, NULL, NULL}
-  };
-
-  if (mode_type == 0)
-    mode_type = g_enum_register_static ("GstAudioCdSrcMode", modes);
-
-  return mode_type;
-}
-
 static void
 gst_audio_cd_src_class_init (GstAudioCdSrcClass * klass)
 {
@@ -243,8 +217,6 @@ gst_audio_cd_src_class_init (GstAudioCdSrcClass * klass)
   GST_DEBUG_CATEGORY_INIT (gst_audio_cd_src_debug, "audiocdsrc", 0,
       "Audio CD source base class");
 
-  g_type_class_add_private (klass, sizeof (GstAudioCdSrcPrivate));
-
   /* our very own formats */
   track_format = gst_format_register ("track", "CD track");
   sector_format = gst_format_register ("sector", "CD sector");
@@ -291,8 +263,8 @@ gst_audio_cd_src_class_init (GstAudioCdSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 #endif
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_audio_cd_src_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_audio_cd_src_src_template);
 
 #if 0
   element_class->set_index = GST_DEBUG_FUNCPTR (gst_audio_cd_src_set_index);
@@ -312,9 +284,7 @@ gst_audio_cd_src_class_init (GstAudioCdSrcClass * klass)
 static void
 gst_audio_cd_src_init (GstAudioCdSrc * src)
 {
-  src->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE (src, GST_TYPE_AUDIO_CD_SRC,
-      GstAudioCdSrcPrivate);
+  src->priv = gst_audio_cd_src_get_instance_private (src);
 
   /* we're not live and we operate in time */
   gst_base_src_set_format (GST_BASE_SRC (src), GST_FORMAT_TIME);
