@@ -1960,6 +1960,10 @@ _create_shader (GstGLColorConvert * convert)
       && info->templ->target != GST_GL_TEXTURE_TARGET_EXTERNAL_OES)
     g_string_append (str, glsl_OES_extension_string);
 
+  g_string_append (str,
+      gst_gl_shader_string_get_highest_precision (convert->context, version,
+          profile));
+
   if (info->templ->uniforms)
     g_string_append (str, info->templ->uniforms);
 
@@ -2036,12 +2040,9 @@ _create_shader (GstGLColorConvert * convert)
       &version, &profile);
   g_free (tmp);
 
-  strings[1] =
-      gst_gl_shader_string_get_highest_precision (convert->context, version,
-      profile);
-  strings[2] = info->frag_prog;
+  strings[1] = info->frag_prog;
   if (!(stage = gst_glsl_stage_new_with_strings (convert->context,
-              GL_FRAGMENT_SHADER, version, profile, 3, strings))) {
+              GL_FRAGMENT_SHADER, version, profile, 2, strings))) {
     GST_ERROR_OBJECT (convert, "Failed to create fragment stage");
     g_free (info->frag_prog);
     info->frag_prog = NULL;
@@ -2290,6 +2291,14 @@ _do_convert_one_view (GstGLContext * context, GstGLColorConvert * convert,
       res = FALSE;
       goto out;
     }
+    if (convert->context != convert->priv->in_tex[i]->mem.context) {
+      GST_ERROR_OBJECT (convert, "input memory OpenGL context is different. "
+          "we have %" GST_PTR_FORMAT " memory has %" GST_PTR_FORMAT,
+          convert->context, convert->priv->in_tex[i]->mem.context);
+      res = FALSE;
+      goto out;
+    }
+
     if (!gst_memory_map ((GstMemory *) convert->priv->in_tex[i], &in_info[i],
             GST_MAP_READ | GST_MAP_GL)) {
       GST_ERROR_OBJECT (convert, "failed to map input memory %p",
@@ -2310,6 +2319,13 @@ _do_convert_one_view (GstGLContext * context, GstGLColorConvert * convert,
       res = FALSE;
       goto out;
     }
+    if (convert->context != out_tex->mem.context) {
+      GST_ERROR_OBJECT (convert, "output memory OpenGL context is different. "
+          "we have %" GST_PTR_FORMAT " memory has %" GST_PTR_FORMAT,
+          convert->context, out_tex->mem.context);
+      res = FALSE;
+      goto out;
+    }
 
     mem_width = gst_gl_memory_get_texture_width (out_tex);
     mem_height = gst_gl_memory_get_texture_height (out_tex);
