@@ -509,6 +509,12 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft MPEG-4 4.3");        /* FIXME? */
       break;
 
+    case GST_MAKE_FOURCC ('C', 'F', 'H', 'D'):
+      caps = gst_caps_new_empty_simple ("video/x-cineform");
+      if (codec_name)
+        *codec_name = g_strdup ("CineForm");
+      break;
+
     case GST_MAKE_FOURCC ('D', 'V', 'S', 'D'):
     case GST_MAKE_FOURCC ('d', 'v', 's', 'd'):
     case GST_MAKE_FOURCC ('d', 'v', 'c', ' '):
@@ -1594,12 +1600,17 @@ gst_riff_create_audio_caps (guint16 codec_id,
       GstMapInfo info;
       gsize size;
 
+      if (strf_data == NULL) {
+        GST_WARNING ("WAVE_FORMAT_EXTENSIBLE but no strf_data buffer provided");
+        return NULL;
+      }
+
       /* should be at least 22 bytes */
       size = gst_buffer_get_size (strf_data);
 
-      if (strf_data == NULL || size < 22) {
+      if (size < 22) {
         GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %" G_GSIZE_FORMAT
-            " (expected: 22)", (strf_data) ? size : -1);
+            " (expected: 22)", size);
         return NULL;
       }
 
@@ -1859,6 +1870,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('3', 'I', 'V', '1'),
     GST_MAKE_FOURCC ('A', 'S', 'V', '1'),
     GST_MAKE_FOURCC ('A', 'S', 'V', '2'),
+    GST_MAKE_FOURCC ('C', 'F', 'H', 'D'),
     GST_MAKE_FOURCC ('C', 'L', 'J', 'R'),
     GST_MAKE_FOURCC ('C', 'S', 'C', 'D'),
     GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'),
