@@ -110,15 +110,11 @@ struct _GstRTPBaseAudioPayloadPrivate
   guint cached_max_length;
   guint cached_ptime_multiple;
   guint cached_align;
+  guint cached_csrc_count;
 
   gboolean buffer_list;
 };
 
-
-#define GST_RTP_BASE_AUDIO_PAYLOAD_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GST_TYPE_RTP_BASE_AUDIO_PAYLOAD, \
-                                GstRTPBaseAudioPayloadPrivate))
-
 static void gst_rtp_base_audio_payload_finalize (GObject * object);
 
 static void gst_rtp_base_audio_payload_set_property (GObject * object,
@@ -158,7 +154,7 @@ static gboolean gst_rtp_base_payload_audio_sink_event (GstRTPBasePayload
     * payload, GstEvent * event);
 
 #define gst_rtp_base_audio_payload_parent_class parent_class
-G_DEFINE_TYPE (GstRTPBaseAudioPayload, gst_rtp_base_audio_payload,
+G_DEFINE_TYPE_WITH_PRIVATE (GstRTPBaseAudioPayload, gst_rtp_base_audio_payload,
     GST_TYPE_RTP_BASE_PAYLOAD);
 
 static void
@@ -168,8 +164,6 @@ gst_rtp_base_audio_payload_class_init (GstRTPBaseAudioPayloadClass * klass)
   GstElementClass *gstelement_class;
   GstRTPBasePayloadClass *gstrtpbasepayload_class;
 
-  g_type_class_add_private (klass, sizeof (GstRTPBaseAudioPayloadPrivate));
-
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstrtpbasepayload_class = (GstRTPBasePayloadClass *) klass;
@@ -198,7 +192,7 @@ gst_rtp_base_audio_payload_class_init (GstRTPBaseAudioPayloadClass * klass)
 static void
 gst_rtp_base_audio_payload_init (GstRTPBaseAudioPayload * payload)
 {
-  payload->priv = GST_RTP_BASE_AUDIO_PAYLOAD_GET_PRIVATE (payload);
+  payload->priv = gst_rtp_base_audio_payload_get_instance_private (payload);
 
   /* these need to be set by child object if frame based */
   payload->frame_size = 0;
@@ -460,7 +454,8 @@ gst_rtp_base_audio_payload_push (GstRTPBaseAudioPayload * baseaudiopayload,
       payload_len, GST_TIME_ARGS (timestamp));
 
   /* create buffer to hold the payload */
-  outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
+  outbuf = gst_rtp_base_payload_allocate_output_buffer (basepayload,
+      payload_len, 0, 0);
 
   /* copy payload */
   gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
@@ -526,7 +521,7 @@ gst_rtp_base_audio_payload_push_buffer (GstRTPBaseAudioPayload *
       payload_len, GST_TIME_ARGS (timestamp));
 
   /* create just the RTP header buffer */
-  outbuf = gst_rtp_buffer_new_allocate (0, 0, 0);
+  outbuf = gst_rtp_base_payload_allocate_output_buffer (basepayload, 0, 0, 0);
 
   /* set metadata */
   gst_rtp_base_audio_payload_set_meta (baseaudiopayload, outbuf, payload_len,
@@ -635,7 +630,7 @@ gst_rtp_base_audio_payload_flush (GstRTPBaseAudioPayload * baseaudiopayload,
 
 
     /* create buffer to hold the payload */
-    outbuf = gst_rtp_buffer_new_allocate (0, 0, 0);
+    outbuf = gst_rtp_base_payload_allocate_output_buffer (basepayload, 0, 0, 0);
 
     paybuf = gst_adapter_take_buffer_fast (adapter, payload_len);
 
@@ -660,8 +655,8 @@ gst_rtp_base_audio_payload_flush (GstRTPBaseAudioPayload * baseaudiopayload,
  * mtu and min/max_ptime values. We cache those so that we don't have to redo
  * all the calculations */
 static gboolean
-gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
-    basepayload, guint * min_payload_len, guint * max_payload_len,
+gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload * basepayload,
+    guint csrc_count, guint * min_payload_len, guint * max_payload_len,
     guint * align)
 {
   GstRTPBaseAudioPayload *payload;
@@ -679,13 +674,16 @@ gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
 
   mtu = GST_RTP_BASE_PAYLOAD_MTU (payload);
 
-  /* check cached values */
+  /* check cached values. Since csrc_count may vary for each packet, we only
+   * check whether the new value exceeds the cached value and thus result in
+   * smaller payload. */
   if (G_LIKELY (priv->cached_mtu == mtu
           && priv->cached_ptime_multiple ==
           basepayload->ptime_multiple
           && priv->cached_ptime == basepayload->ptime
           && priv->cached_max_ptime == basepayload->max_ptime
-          && priv->cached_min_ptime == basepayload->min_ptime)) {
+          && priv->cached_min_ptime == basepayload->min_ptime
+          && priv->cached_csrc_count >= csrc_count)) {
     /* if nothing changed, return cached values */
     *min_payload_len = priv->cached_min_length;
     *max_payload_len = priv->cached_max_length;
@@ -704,7 +702,7 @@ gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
     maxptime_octets = G_MAXUINT;
   }
   /* MTU max */
-  max_mtu = gst_rtp_buffer_calc_payload_len (mtu, 0, 0);
+  max_mtu = gst_rtp_buffer_calc_payload_len (mtu, 0, csrc_count);
   /* round down to alignment */
   max_mtu = ALIGN_DOWN (max_mtu, *align);
 
@@ -740,6 +738,7 @@ gst_rtp_base_audio_payload_get_lengths (GstRTPBasePayload *
   priv->cached_min_length = *min_payload_len;
   priv->cached_max_length = *max_payload_len;
   priv->cached_align = *align;
+  priv->cached_csrc_count = csrc_count;
 
   return TRUE;
 }
@@ -882,8 +881,9 @@ gst_rtp_base_audio_payload_handle_buffer (GstRTPBasePayload *
     }
   }
 
-  if (!gst_rtp_base_audio_payload_get_lengths (basepayload, &min_payload_len,
-          &max_payload_len, &align))
+  if (!gst_rtp_base_audio_payload_get_lengths (basepayload,
+          gst_rtp_base_payload_get_source_count (basepayload, buffer),
+          &min_payload_len, &max_payload_len, &align))
     goto config_error;
 
   GST_DEBUG_OBJECT (payload,
