@@ -14,24 +14,21 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
  * SECTION:gstvideosink
+ * @title: GstVideoSink
  * @short_description: Base class for video sinks
- * 
- * <refsect2>
- * <para>
- * Provides useful functions and a base class for video sinks. 
- * </para>
- * <para>
+ *
+ * Provides useful functions and a base class for video sinks.
+ *
  * GstVideoSink will configure the default base sink to drop frames that
  * arrive later than 20ms as this is considered the default threshold for
  * observing out-of-sync frames.
- * </para>
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,8 +49,27 @@ struct _GstVideoSinkPrivate
   gboolean show_preroll_frame;  /* ATOMIC */
 };
 
-GST_DEBUG_CATEGORY_STATIC (video_sink_debug);
-#define GST_CAT_DEFAULT video_sink_debug
+G_DEFINE_TYPE_WITH_PRIVATE (GstVideoSink, gst_video_sink, GST_TYPE_BASE_SINK);
+
+#ifndef GST_DISABLE_GST_DEBUG
+#define GST_CAT_DEFAULT gst_video_sink_ensure_debug_category()
+
+static GstDebugCategory *
+gst_video_sink_ensure_debug_category (void)
+{
+  static gsize cat_gonce = 0;
+
+  if (g_once_init_enter (&cat_gonce)) {
+    GstDebugCategory *cat = NULL;
+
+    GST_DEBUG_CATEGORY_INIT (cat, "videosink", 0, "GstVideoSink");
+
+    g_once_init_leave (&cat_gonce, (gsize) cat);
+  }
+
+  return (GstDebugCategory *) cat_gonce;
+}
+#endif /* GST_DISABLE_GST_DEBUG */
 
 static GstBaseSinkClass *parent_class = NULL;
 
@@ -73,7 +89,7 @@ static GstFlowReturn gst_video_sink_show_frame (GstBaseSink * bsink,
  * @dst: the #GstVideoRectangle describing the destination area
  * @result: a pointer to a #GstVideoRectangle which will receive the result area
  * @scaling: a #gboolean indicating if scaling should be applied or not
- * 
+ *
  * Takes @src rectangle and position it at the center of @dst rectangle with or
  * without @scaling. It handles clipping if the @src rectangle is bigger than
  * the @dst one and @scaling is set to FALSE.
@@ -87,8 +103,8 @@ gst_video_sink_center_rect (GstVideoRectangle src, GstVideoRectangle dst,
   if (!scaling) {
     result->w = MIN (src.w, dst.w);
     result->h = MIN (src.h, dst.h);
-    result->x = (dst.w - result->w) / 2;
-    result->y = (dst.h - result->h) / 2;
+    result->x = dst.x + (dst.w - result->w) / 2;
+    result->y = dst.y + (dst.h - result->h) / 2;
   } else {
     gdouble src_ratio, dst_ratio;
 
@@ -98,16 +114,16 @@ gst_video_sink_center_rect (GstVideoRectangle src, GstVideoRectangle dst,
     if (src_ratio > dst_ratio) {
       result->w = dst.w;
       result->h = dst.w / src_ratio;
-      result->x = 0;
-      result->y = (dst.h - result->h) / 2;
+      result->x = dst.x;
+      result->y = dst.y + (dst.h - result->h) / 2;
     } else if (src_ratio < dst_ratio) {
       result->w = dst.h * src_ratio;
       result->h = dst.h;
-      result->x = (dst.w - result->w) / 2;
-      result->y = 0;
+      result->x = dst.x + (dst.w - result->w) / 2;
+      result->y = dst.y;
     } else {
-      result->x = 0;
-      result->y = 0;
+      result->x = dst.x;
+      result->y = dst.y;
       result->w = dst.w;
       result->h = dst.h;
     }
@@ -126,11 +142,12 @@ gst_video_sink_init (GstVideoSink * videosink)
   videosink->height = 0;
 
   /* 20ms is more than enough, 80-130ms is noticable */
-  gst_base_sink_set_max_lateness (GST_BASE_SINK (videosink), 20 * GST_MSECOND);
+  gst_base_sink_set_processing_deadline (GST_BASE_SINK (videosink),
+      15 * GST_MSECOND);
+  gst_base_sink_set_max_lateness (GST_BASE_SINK (videosink), 5 * GST_MSECOND);
   gst_base_sink_set_qos_enabled (GST_BASE_SINK (videosink), TRUE);
 
-  videosink->priv = G_TYPE_INSTANCE_GET_PRIVATE (videosink,
-      GST_TYPE_VIDEO_SINK, GstVideoSinkPrivate);
+  videosink->priv = gst_video_sink_get_instance_private (videosink);
 }
 
 static void
@@ -147,7 +164,7 @@ gst_video_sink_class_init (GstVideoSinkClass * klass)
   /**
    * GstVideoSink:show-preroll-frame:
    *
-   * Whether to show video frames during preroll. If set to #FALSE, video
+   * Whether to show video frames during preroll. If set to %FALSE, video
    * frames will only be rendered in PLAYING state.
    */
   g_object_class_install_property (gobject_class, PROP_SHOW_PREROLL_FRAME,
@@ -159,14 +176,6 @@ gst_video_sink_class_init (GstVideoSinkClass * klass)
   basesink_class->render = GST_DEBUG_FUNCPTR (gst_video_sink_show_frame);
   basesink_class->preroll =
       GST_DEBUG_FUNCPTR (gst_video_sink_show_preroll_frame);
-
-  g_type_class_add_private (klass, sizeof (GstVideoSinkPrivate));
-}
-
-static void
-gst_video_sink_base_init (gpointer g_class)
-{
-  GST_DEBUG_CATEGORY_INIT (video_sink_debug, "videosink", 0, "GstVideoSink");
 }
 
 static GstFlowReturn
@@ -257,30 +266,3 @@ gst_video_sink_get_property (GObject * object, guint prop_id,
       break;
   }
 }
-
-/* Public methods */
-
-GType
-gst_video_sink_get_type (void)
-{
-  static GType videosink_type = 0;
-
-  if (!videosink_type) {
-    static const GTypeInfo videosink_info = {
-      sizeof (GstVideoSinkClass),
-      gst_video_sink_base_init,
-      NULL,
-      (GClassInitFunc) gst_video_sink_class_init,
-      NULL,
-      NULL,
-      sizeof (GstVideoSink),
-      0,
-      (GInstanceInitFunc) gst_video_sink_init,
-    };
-
-    videosink_type = g_type_register_static (GST_TYPE_BASE_SINK,
-        "GstVideoSink", &videosink_info, 0);
-  }
-
-  return videosink_type;
-}
