@@ -72,6 +72,9 @@ GstPad *         gst_play_sink_request_pad    (GstPlaySink *playsink, GstPlaySin
 void             gst_play_sink_release_pad    (GstPlaySink *playsink, GstPad *pad);
 void             gst_play_sink_refresh_pad    (GstPlaySink *playsink, GstPad *pad, GstPlaySinkType type);
 
+void             gst_play_sink_set_filter     (GstPlaySink * playsink, GstPlaySinkType type, GstElement * filter);
+GstElement *     gst_play_sink_get_filter     (GstPlaySink * playsink, GstPlaySinkType type);
+
 void             gst_play_sink_set_sink       (GstPlaySink * playsink, GstPlaySinkType type, GstElement * sink);
 GstElement *     gst_play_sink_get_sink       (GstPlaySink * playsink, GstPlaySinkType type);
 
@@ -95,6 +98,9 @@ gchar *          gst_play_sink_get_subtitle_encoding  (GstPlaySink *playsink);
 void             gst_play_sink_set_av_offset  (GstPlaySink *playsink, gint64 av_offset);
 gint64           gst_play_sink_get_av_offset  (GstPlaySink *playsink);
 
+void             gst_play_sink_set_text_offset  (GstPlaySink *playsink, gint64 text_offset);
+gint64           gst_play_sink_get_text_offset  (GstPlaySink *playsink);
+
 GstSample *      gst_play_sink_get_last_sample (GstPlaySink * playsink);
 GstSample *      gst_play_sink_convert_sample  (GstPlaySink * playsink, GstCaps * caps);
 
