@@ -28,12 +28,14 @@ G_BEGIN_DECLS
 #define DEFAULT_RAW_CAPS \
     "video/x-raw(ANY); " \
     "audio/x-raw(ANY); " \
-    "text/x-raw; " \
+    "text/x-raw(ANY); " \
     "subpicture/x-dvd; " \
     "subpicture/x-dvb; " \
-    "subpicture/x-pgs"
+    "subpicture/x-xsub; " \
+    "subpicture/x-pgs; " \
+    "closedcaption/x-cea-608; " \
+    "closedcaption/x-cea-708"
 
 G_END_DECLS
 
 #endif /* __GST_RAW_CAPS__ */
-
