@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -78,7 +78,7 @@ typedef enum
  * @GST_SYNC_METHOD_NEXT_KEYFRAME       : client receives next keyframe
  * @GST_SYNC_METHOD_LATEST_KEYFRAME     : client receives latest keyframe (burst)
  * @GST_SYNC_METHOD_BURST               : client receives specific amount of data
- * @GST_SYNC_METHOD_BURST_KEYFRAME      : client receives specific amount of data 
+ * @GST_SYNC_METHOD_BURST_KEYFRAME      : client receives specific amount of data
  *                                        starting from latest keyframe
  * @GST_SYNC_METHOD_BURST_WITH_KEYFRAME : client receives specific amount of data from
  *                                        a keyframe, or if there is not enough data after
@@ -202,9 +202,6 @@ struct _GstMultiHandleSink {
   GMainContext *main_context;
   GCancellable *cancellable;
 
-  GSList *streamheader; /* GSList of GstBuffers to use as streamheader */
-  gboolean previous_buffer_in_caps;
-
   gint qos_dscp;
 
   GArray *bufqueue;     /* global queue of buffers */
