@@ -29,6 +29,7 @@
 #include <glib/gstdio.h>
 #include <glib/gprintf.h>
 
+static gboolean have_theora, have_ogg;
 
 GST_START_TEST (test_disco_init)
 {
@@ -56,7 +57,7 @@ GST_START_TEST (test_disco_serializing)
       g_build_filename (GST_TEST_FILES_PATH, "theora-vorbis.ogg", NULL);
 
   /* high timeout, in case we're running under valgrind */
-  dc = gst_discoverer_new (5 * GST_SECOND, &err);
+  dc = gst_discoverer_new (30 * GST_SECOND, &err);
   fail_unless (dc != NULL);
   fail_unless (err == NULL);
 
@@ -66,6 +67,15 @@ GST_START_TEST (test_disco_serializing)
 
   info = gst_discoverer_discover_uri (dc, uri, &err);
   fail_unless (info);
+  if (have_theora && have_ogg) {
+    fail_unless_equals_int (gst_discoverer_info_get_result (info),
+        GST_DISCOVERER_OK);
+  } else {
+    fail_unless_equals_int (gst_discoverer_info_get_result (info),
+        GST_DISCOVERER_MISSING_PLUGINS);
+    g_clear_error (&err);
+    goto missing_plugins;
+  }
   serialized =
       gst_discoverer_info_to_variant (info, GST_DISCOVERER_SERIALIZE_ALL);
 
@@ -83,13 +93,15 @@ GST_START_TEST (test_disco_serializing)
 
   fail_unless (g_variant_equal (serialized, reserialized));
 
-  gst_discoverer_info_unref (info);
   gst_discoverer_info_unref (dinfo);
-  g_free (uri);
   g_variant_unref (serialized);
   g_variant_unref (reserialized);
 
+missing_plugins:
+
+  gst_discoverer_info_unref (info);
   g_object_unref (dc);
+  g_free (uri);
 }
 
 GST_END_TEST;
@@ -103,7 +115,7 @@ GST_START_TEST (test_disco_sync)
   gchar *uri;
 
   /* high timeout, in case we're running under valgrind */
-  dc = gst_discoverer_new (5 * GST_SECOND, &err);
+  dc = gst_discoverer_new (30 * GST_SECOND, &err);
   fail_unless (dc != NULL);
   fail_unless (err == NULL);
 
@@ -219,7 +231,9 @@ GST_START_TEST (test_disco_missing_plugins)
     result = gst_discoverer_info_get_result (info);
     GST_INFO ("result: %d, error message: %s", result, err->message);
     fail_unless_equals_int (result, GST_DISCOVERER_MISSING_PLUGINS);
+#ifndef GST_DISABLE_DEPRECATED
     GST_INFO ("misc: %" GST_PTR_FORMAT, gst_discoverer_info_get_misc (info));
+#endif
 
     gst_discoverer_info_unref (info);
     g_error_free (err);
@@ -231,12 +245,145 @@ GST_START_TEST (test_disco_missing_plugins)
 
 GST_END_TEST;
 
+typedef struct _AsyncTestData
+{
+  gchar *uri;
+  GMainLoop *loop;
+  GstDiscovererResult result;
+} AsyncTestData;
+
+static void
+discovered_cb (GstDiscoverer * discoverer,
+    GstDiscovererInfo * info, GError * err, AsyncTestData * data)
+{
+  const gchar *uri = gst_discoverer_info_get_uri (info);
+
+  fail_unless_equals_string (data->uri, uri);
+
+  /* cannot ensure GST_DISCOVERER_OK since there might be missing plugins */
+  data->result = gst_discoverer_info_get_result (info);
+
+  g_main_loop_quit (data->loop);
+}
+
+static void
+test_disco_async_with_context (GMainContext * context)
+{
+  GstDiscoverer *dc;
+  GError *err = NULL;
+  AsyncTestData data = { 0, };
+  gchar *path =
+      g_build_filename (GST_TEST_FILES_PATH, "theora-vorbis.ogg", NULL);
+
+  if (context)
+    g_main_context_push_thread_default (context);
+
+  data.uri = gst_filename_to_uri (path, &err);
+  /* something wrong if we have error here */
+  fail_unless (err == NULL);
+  g_free (path);
+
+  data.loop = g_main_loop_new (context, FALSE);
+
+  /* high timeout, in case we're running under valgrind */
+  dc = gst_discoverer_new (30 * GST_SECOND, &err);
+  fail_unless (dc != NULL);
+  fail_unless (err == NULL);
+
+  g_signal_connect (dc, "discovered", G_CALLBACK (discovered_cb), &data);
+
+  gst_discoverer_start (dc);
+  fail_unless (gst_discoverer_discover_uri_async (dc, data.uri) == TRUE);
+
+  g_main_loop_run (data.loop);
+
+  if (have_theora && have_ogg) {
+    fail_unless_equals_int (data.result, GST_DISCOVERER_OK);
+  } else {
+    fail_unless_equals_int (data.result, GST_DISCOVERER_MISSING_PLUGINS);
+  }
+
+  gst_discoverer_stop (dc);
+  g_object_unref (dc);
+  g_free (data.uri);
+
+  g_main_loop_unref (data.loop);
+
+  if (context)
+    g_main_context_pop_thread_default (context);
+}
+
+GST_START_TEST (test_disco_async)
+{
+  /* use default GMainContext */
+  test_disco_async_with_context (NULL);
+}
+
+GST_END_TEST;
+
+typedef struct _CustomContextData
+{
+  GMutex lock;
+  GCond cond;
+  gboolean finish;
+} CustomContextData;
+
+
+static gpointer
+custom_context_thread_func (CustomContextData * data)
+{
+  GMainContext *context;
+
+  /* test async APIs with custom GMainContext */
+  context = g_main_context_new ();
+  test_disco_async_with_context (context);
+  g_main_context_unref (context);
+
+  data->finish = TRUE;
+  g_cond_signal (&data->cond);
+
+  return NULL;
+}
+
+GST_START_TEST (test_disco_async_custom_context)
+{
+  GThread *thread;
+  CustomContextData data;
+
+  g_mutex_init (&data.lock);
+  g_cond_init (&data.cond);
+  data.finish = FALSE;
+
+  /* ensure default context here, but we will use other thread default context
+   * instead of this */
+  g_main_context_default ();
+
+  thread = g_thread_new ("test-custom-context-thread",
+      (GThreadFunc) custom_context_thread_func, &data);
+
+  g_mutex_lock (&data.lock);
+  while (!data.finish)
+    g_cond_wait (&data.cond, &data.lock);
+  g_mutex_unlock (&data.lock);
+
+  g_thread_join (thread);
+  g_mutex_clear (&data.lock);
+  g_cond_clear (&data.cond);
+}
+
+GST_END_TEST;
+
 static Suite *
 discoverer_suite (void)
 {
   Suite *s = suite_create ("discoverer");
   TCase *tc_chain = tcase_create ("general");
 
+  have_theora = gst_registry_check_feature_version (gst_registry_get (),
+      "theoradec", GST_VERSION_MAJOR, GST_VERSION_MINOR, 0);
+  have_ogg = gst_registry_check_feature_version (gst_registry_get (),
+      "oggdemux", GST_VERSION_MAJOR, GST_VERSION_MINOR, 0);
+
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_disco_init);
   tcase_add_test (tc_chain, test_disco_sync);
@@ -245,6 +392,8 @@ discoverer_suite (void)
   tcase_add_test (tc_chain, test_disco_sync_reuse_timeout);
   tcase_add_test (tc_chain, test_disco_missing_plugins);
   tcase_add_test (tc_chain, test_disco_serializing);
+  tcase_add_test (tc_chain, test_disco_async);
+  tcase_add_test (tc_chain, test_disco_async_custom_context);
   return s;
 }
 
